@@ -89,7 +89,8 @@ static int logitech_command(int fd, char *c)
 	unsigned char d;
 
 	for (i = 0; c[i]; i++) {
-		write(fd, c + i, 1);
+		if (write(fd, c + i, 1) != 1)
+			return -1;
 		if (readchar(fd, &d, 1000))
 			return -1;
 		if (c[i] != d)
@@ -100,7 +101,8 @@ static int logitech_command(int fd, char *c)
 
 static int magellan_init(int fd, unsigned long *id, unsigned long *extra)
 {
-	write(fd, "m3\rpBB\rz\r", 9);
+	if (write(fd, "m3\rpBB\rz\r", 9) != 9)
+		return -1;
 	return 0;
 }
 
@@ -150,8 +152,10 @@ static int spaceball_cmd(int fd, char *c, char *d)
 	int i;
 
 	for (i = 0; c[i]; i++)
-		write(fd, c + i, 1);
-	write(fd, "\r", 1);
+		if (write(fd, c + i, 1) != 1)
+			return -1;
+	if (write(fd, "\r", 1) != 1)
+		return -1;
 
 	i = spaceball_waitcmd(fd, toupper(c[0]), d);
 
@@ -532,6 +536,33 @@ static int zhenhua_init(int fd, unsigned long *id, unsigned long *extra)
 		
 }
 
+#define EP_PROMPT_MODE  "B"     /* Prompt mode */
+#define EP_ABSOLUTE     "F"     /* Absolute Mode */
+#define EP_UPPER_ORIGIN "b"     /* Origin upper left */
+#define EP_STREAM_MODE  "@"     /* Stream mode */
+
+static int easypen_init(int fd, unsigned long *id, unsigned long *extra)
+{
+	char buf[256];
+
+	/* reset */
+	write(fd, 0, 1);
+	usleep(400000);
+
+	/* set prompt mode */
+	if (write(fd, EP_PROMPT_MODE, 1) == -1)
+		return -1;
+
+	/* clear buffer */
+	while (read(fd, buf, sizeof(buf)) == sizeof(buf));
+
+	/* set options */
+	if (write(fd, EP_ABSOLUTE EP_STREAM_MODE EP_UPPER_ORIGIN, 3) == -1)
+		return -1;
+
+	return 0;
+}
+
 static int dump_init(int fd, unsigned long *id, unsigned long *extra)
 {
 	unsigned char c, o = 0;
@@ -636,7 +667,7 @@ static struct input_types input_types[] = {
 	B2400, CS8,
 	SERIO_TWIDJOY,		0x00,	0x00,	0,	twiddler_init },
 { "--elotouch",		"-elo",		"ELO touchscreen, 10-byte mode",
-	B9600, CS8 | CRTSCTS,
+	B9600, CS8,
 	SERIO_ELO,		0x00,	0x00,	0,	NULL },
 { "--elo4002",		"-elo6b",	"ELO touchscreen, 6-byte mode",
 	B9600, CS8 | CRTSCTS,
@@ -650,9 +681,11 @@ static struct input_types input_types[] = {
 { "--mtouch",		"-mtouch",	"MicroTouch (3M) touchscreen",
 	B9600, CS8 | CRTSCTS,
 	SERIO_MICROTOUCH,	0x00,	0x00,	0,	NULL },
+#ifdef SERIO_TSC40
 { "--tsc",		"-tsc",		"TSC-10/25/40 serial touchscreen",
 	B9600, CS8,
 	SERIO_TSC40,		0x00,	0x00,	0,	tsc40_init },
+#endif
 { "--touchit213",	"-t213",	"Sahara Touch-iT213 Tablet PC",
 	B9600, CS8,
 	SERIO_TOUCHIT213,	0x00,	0x00,	0,	t213_init },
@@ -683,6 +716,9 @@ static struct input_types input_types[] = {
 { "--zhen-hua",		"-zhen",	"Zhen Hua 5-byte protocol",
 	B19200, CS8,
 	SERIO_ZHENHUA,		0x00,	0x00,	0,	zhenhua_init },
+{ "--easypen",		"-ep",		"Genius EasyPen 3x4 tablet",
+	B9600, CS8|CREAD|CLOCAL|HUPCL|PARENB|PARODD,
+	SERIO_EASYPEN,		0x00,	0x00,	0,	easypen_init },
 #ifdef SERIO_TAOSEVM
 { "--taos-evm",		"-taos",	"TAOS evaluation module",
 	B1200, CS8,
@@ -732,7 +768,6 @@ int main(int argc, char **argv)
 	int baud = -1;
 	int ignore_init_res = 0;
 	int no_init = 0;
-	int one_read = 0;
 
 	for (i = 1; i < argc; i++) {
 		if (!strcasecmp(argv[i], "--help")) {
@@ -848,18 +883,13 @@ int main(int argc, char **argv)
 		retval = EXIT_FAILURE;
 	}
 
+	errno = 0;
 	do {
 		i = read(fd, NULL, 0);
-		if (i == -1) {
-			if (RETRY_ERROR(errno))
-				continue;
-		} else {
-			one_read = 1;
-		}
-	} while (!i);
+	} while (RETRY_ERROR(errno));
 
 	ldisc = 0;
-	if (one_read) {
+	if (errno == 0) {
 		// If we've never managed to read, avoid resetting the line
 		// discipline - another inputattach is probably running
 		ioctl(fd, TIOCSETD, &ldisc);
